@@ -18,7 +18,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
  * 02111-1307 USA
  *
- * $Id: load_stats.c,v 1.17 2004/11/07 12:29:20 ats Exp $ 
+ * $Id: load_stats.c,v 1.18 2005/09/24 13:29:22 tdb Exp $ 
  */
 
 #ifdef HAVE_CONFIG_H
@@ -42,6 +42,7 @@
 
 sg_load_stats *sg_get_load_stats(){
 
+#if !defined(CYGWIN) && !defined(WIN32)
 	static sg_load_stats load_stat;
 
 #ifdef HPUX
@@ -49,10 +50,14 @@ sg_load_stats *sg_get_load_stats(){
 #else
 	double loadav[3];
 #endif
+#endif /* not CYGWIN or WIN32 */
 
 #ifdef CYGWIN
 	sg_set_error(SG_ERROR_UNSUPPORTED, "Cygwin");
 	return NULL;
+#elif defined(WIN32)
+	sg_set_error(SG_ERROR_UNSUPPORTED, "Win32");
+	return NULL;
 #else
 
 #if defined(SOLARIS) && !defined(HAVE_SYS_LOADAVG_H)
