@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  *
- * $Id: saidar.c,v 1.33 2005/01/17 16:34:26 tdb Exp $
+ * $Id: saidar.c,v 1.34 2005/09/07 14:15:40 tdb Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -335,8 +335,11 @@ void display_data(){
 		r=0;
 		w=0;
 		for(counter=0;counter<stats.disk_io_entries;counter++){
+			char name[12];
+			strncpy(name, disk_io_stat_ptr->disk_name, sizeof(name));
+			name[sizeof(name)-1] = '\0'; /* strncpy doesn't terminate longer strings */
 			move(line, 0);
-			printw("%s", disk_io_stat_ptr->disk_name);
+			printw("%s", name);
 			move(line, 12);
 			rt = (disk_io_stat_ptr->systime)? (disk_io_stat_ptr->read_bytes/disk_io_stat_ptr->systime): disk_io_stat_ptr->read_bytes;
 			printw("%7s", size_conv(rt));
@@ -362,8 +365,11 @@ void display_data(){
 		/* Network */
 		network_stat_ptr = stats.network_io_stats;
 		for(counter=0;counter<stats.network_io_entries;counter++){
+			char name[20];
+			strncpy(name, network_stat_ptr->interface_name, sizeof(name));
+			name[sizeof(name)-1] = '\0'; /* strncpy doesn't terminate longer strings */
 			move(line, 42);
-			printw("%s", network_stat_ptr->interface_name);
+			printw("%s", name);
 			move(line, 62);
 			rt = (network_stat_ptr->systime)? (network_stat_ptr->rx / network_stat_ptr->systime): network_stat_ptr->rx;
 			printw("%7s", size_conv(rt));
@@ -380,8 +386,11 @@ void display_data(){
 		/* Disk */
 		disk_stat_ptr = stats.fs_stats;
 		for(counter=0;counter<stats.fs_entries;counter++){
+			char name[20];
+			strncpy(name, disk_stat_ptr->mnt_point, sizeof(name));
+			name[sizeof(name)-1] = '\0'; /* strncpy doesn't terminate longer strings */
 			move(line, 42);
-			printw("%s", disk_stat_ptr->mnt_point);
+			printw("%s", name);
 			move(line, 62);
 			printw("%7s", size_conv(disk_stat_ptr->avail));
 			move(line, 73);
